#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#ifdef HAVE_DIRAC
#include "mp_msg.h"

#include "vd_internal.h"

static vd_info_t info = 
{
	"Dirac Video decoder v0.9.0",
	"dirac",
	"Dirac <dirac at rd dot bbc dot co dot uk>",
	"Dirac",
	"native"
};

LIBVD_EXTERN(dirac)
#include <libdirac_decoder/dirac_parser.h>

// to set/get/query special features/parameters
static int control(sh_video_t *sh,int cmd,void* arg,...)
{
	return CONTROL_UNKNOWN;
}

// init driver
static int init(sh_video_t *sh)
{
	dirac_decoder_t *diracdec ;

	/*
	* initialise cpu acceleration stuff when we have it in Dirac
	*/

	/*
	* initialise decoder
	*/
	diracdec = dirac_decoder_init(verbose > 4);

	if (!diracdec)
		return 0;

	sh->context = diracdec;

	return 1;
}

// uninit driver
static void uninit(sh_video_t *sh)
{
	dirac_decoder_t *diracdec = sh->context;
	if (diracdec)
		dirac_decoder_close (diracdec);
}

// decode a frame
static mp_image_t* decode(sh_video_t *sh,void* data,int len,int flags)
{
	dirac_decoder_t *diracdec = sh->context;
	mp_image_t* mpi=NULL;
	int drop_frame, framedrop=flags&3;
	unsigned char *buf[3];


	if(len<=0) return 0; // skipped null frame

	dirac_buffer (diracdec, data, data+len);

	while (1)
	{
		DecoderState state = dirac_parse (diracdec);

		switch (state)
		{
		case STATE_BUFFER:
			return 0;

		case STATE_SEQUENCE:
			mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"Dirac: STATE_SEQUENCE width=%d height=%d\n", diracdec->src_params.width, diracdec->src_params.height);
			if (diracdec->src_params.chroma == format422)
			{
				if(!mpcodecs_config_vo(sh, 
					diracdec->src_params.width, 
					diracdec->src_params.height, 
					IMGFMT_422P))
					return 0;
			}
			else
			{
				if(!mpcodecs_config_vo(sh, 
					diracdec->src_params.width, 
					diracdec->src_params.height, 
					IMGFMT_I420))
					return 0;
			}
			sh->disp_w = diracdec->src_params.width;
			sh->disp_h = diracdec->src_params.height;
			sh->format = 0x20000001;
			sh->fps = diracdec->src_params.frame_rate.numerator / 
		              diracdec->src_params.frame_rate.denominator;
			buf[0] = (unsigned char *)malloc (diracdec->src_params.width * diracdec->src_params.height);
			buf[1] = (unsigned char *)malloc (diracdec->src_params.chroma_width * diracdec->src_params.chroma_height);
			buf[2] = (unsigned char *)malloc (diracdec->src_params.chroma_width * diracdec->src_params.chroma_height);

			if (!buf[0] || !buf[1] || !buf[2])
			{
				free(buf[0]);
				free(buf[1]);
				free(buf[2]);
				return 0;
			}
			dirac_set_buf(diracdec, buf, mpi);
			break;
	
		case STATE_SEQUENCE_END:
			mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"Dirac: STATE_SEQUENCE_END\n");
			free (diracdec->fbuf->buf[0]);
			free (diracdec->fbuf->buf[1]);
			free (diracdec->fbuf->buf[2]);
			break;
	
		case STATE_PICTURE_AVAIL:
			mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"Dirac: STATE_PICTURE_AVAIL fnum=%d ftype=%s reftype=%s\n", 
			diracdec->frame_params.fnum, 
			diracdec->frame_params.ftype == INTER_FRAME ? "INTER" : 
					"INTRA_FRAME",
			diracdec->frame_params.rtype == REFERENCE_FRAME ? "REF" : 
					"NON_REF");
			mpi=mpcodecs_get_image(sh, MP_IMGTYPE_EXPORT, 0, sh->disp_w, sh->disp_h);
			if (mpi)
			{
				mpi->planes[0] = diracdec->fbuf->buf[0];
				mpi->stride[0] = diracdec->src_params.width;
				mpi->planes[1] = diracdec->fbuf->buf[1];
				mpi->stride[1] = diracdec->src_params.chroma_width;
				mpi->planes[2] = diracdec->fbuf->buf[2];
				mpi->stride[2] = diracdec->src_params.chroma_width;
				return mpi;
			}
		case STATE_PICTURE_START:
			mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"Dirac: STATE_PICTURE_START fnum=%d ftype=%s\n", 
			diracdec->frame_params.fnum, 
			diracdec->frame_params.ftype == INTER_FRAME ? "INTER" : 
					"INTRA_FRAME",
			diracdec->frame_params.rtype == REFERENCE_FRAME ? "REF" : 
					"NON_REF");
			sh->pts =  (diracdec->frame_params.fnum+1)*sh->frametime;
			drop_frame = framedrop && (diracdec->frame_params.ftype == INTER_FRAME && diracdec->frame_params.rtype == NON_REFERENCE_FRAME);
			drop_frame |= framedrop>=2;
			if (drop_frame)
			{
				mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"      Skipping frame\n");
				dirac_skip(diracdec, 1);
			}
			else
				dirac_skip(diracdec, 0);
			break;
		
		case STATE_INVALID:
			mp_msg(MSGT_DECVIDEO,MSGL_DBG2,"Dirac: STATE_INVALID");
			free (diracdec->fbuf->buf[0]);
			free (diracdec->fbuf->buf[1]);
			free (diracdec->fbuf->buf[2]);
			return 0;
			break;

		default:
			break;
		}
	}
	return 0;
}

#endif
