#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include "config.h"

#ifdef HAVE_DIRAC
#include "mp_msg.h"
#include "help_mp.h"

#include "stream/stream.h"
#include "demuxer.h"
#include "stheader.h"

#include <libdirac_decoder/dirac_parser.h>



static int dirac_check_file(struct demuxer_st *demuxer)
{
	unsigned char tmp_buffer[4096];
	int bytes_read;
	dirac_decoder_t *diracdec;
	DecoderState state;
	int ret_stat = 0;

	mp_msg(MSGT_DEMUX,MSGL_V,"Checking for Dirac\n");
	diracdec = dirac_decoder_init(verbose);

	if (diracdec ==NULL)
		return 0;

	while(1)
	{
		bytes_read = stream_read(demuxer->stream,tmp_buffer, 4096);
		if (bytes_read == 0)
			break;
		dirac_buffer (diracdec, tmp_buffer, tmp_buffer + bytes_read);

		state = dirac_parse (diracdec);
		switch (state)
		{
		case STATE_BUFFER:
			break;

		case STATE_SEQUENCE:
			ret_stat = DEMUXER_TYPE_DIRAC;
			break;

		default:
			break;

		}
		if (ret_stat)
			break;
	}

	dirac_decoder_close(diracdec);
	stream_reset(demuxer->stream);
	stream_seek(demuxer->stream, 0);

	return ret_stat;
}

static struct demuxer_st* demux_open_dirac(struct demuxer_st *demuxer)
{
	unsigned char tmp_buffer[4096];
	int bytes_read;
	dirac_decoder_t *diracdec;
	DecoderState state;
	int ret_stat = 0;

	mp_msg(MSGT_DEMUX,MSGL_V,"Opening Dirac stream\n");
	diracdec = dirac_decoder_init(verbose);
	if (diracdec ==NULL)
		return NULL;

	stream_reset(demuxer->stream);
	stream_seek(demuxer->stream, 0);
	

	while(1)
	{
		bytes_read = stream_read(demuxer->stream,tmp_buffer, 4096);
		if (bytes_read == 0)
			break;
		dirac_buffer (diracdec, tmp_buffer, tmp_buffer + bytes_read);

		state = dirac_parse (diracdec);
		switch (state)
		{
		case STATE_BUFFER:
			break;

		case STATE_SEQUENCE:
			ret_stat = 1;
			break;

		default:
			break;

		}
		if (ret_stat)
			break;
	}
	if (ret_stat)
	{
		sh_video_t *sh_video = new_sh_video(demuxer, 0);
		/* FIXME: set seekable to 1 after including code for seek */
		demuxer->seekable = 0;
		demuxer->video->sh = sh_video;
		sh_video->ds = demuxer->video;
		/* FIXME: defined in codecs.conf. Do we need to get a unique
		 * number from somebody!!!
		 */
		sh_video->format = 0x20000001;
		sh_video->disp_w = diracdec->src_params.width;
		sh_video->disp_h = diracdec->src_params.height;
		sh_video->fps = diracdec->src_params.frame_rate.numerator / 
		                diracdec->src_params.frame_rate.denominator;
		sh_video->frametime = 1.0/sh_video->fps;
	}

	dirac_decoder_close(diracdec);
	stream_reset(demuxer->stream);
	stream_seek(demuxer->stream, 0);

	return demuxer;
}

static int demux_dirac_fill_buffer(struct demuxer_st *demuxer, demux_stream_t *dsds)
{
	demux_packet_t* dp_video=NULL;
	unsigned char tmp_buffer[4096];

	int bytes_read;
	bytes_read = stream_read(demuxer->stream,tmp_buffer, 4096);
	if (bytes_read)
	{
		dp_video=new_demux_packet(bytes_read);
		memmove(dp_video->buffer, tmp_buffer, bytes_read);
		dp_video->flags=0;
		dp_video->pos=stream_tell(demuxer->stream);
		ds_add_packet(demuxer->video,dp_video);
		return 1;
	}
	return 0;
}

static void demux_close_dirac(struct demuxer_st* demuxer)
{
	return;
}

demuxer_desc_t demuxer_desc_dirac = {
    "Dirac Demuxer",
	"dirac",
	"Dirac",
	"Anuradha Suraparaju",
	"",
	DEMUXER_TYPE_DIRAC,
	0,
	NULL,
	demux_dirac_fill_buffer,
	demux_open_dirac,
	demux_close_dirac,
	NULL,
	NULL
};
#endif
