#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "config.h"
#include "mp_msg.h"
#include "help_mp.h"

#include "stream/stream.h"
#include "demuxer.h"
#include "parse_dirac.h"

#define DIRAC_VIDEOBUFFER_SIZE 0x100000

static unsigned char* dirac_videobuffer = 0;
static int dirac_videobuf_len =0;
static unsigned char dirac_videobuf_code[5];
static int dirac_videobuf_code_len = 0;

int dirac_sync_video_packet (demux_stream_t *ds)
{
	int skipped = 0;
	dirac_videobuf_len = 0;
	while(dirac_videobuf_code_len<5)
	{
		dirac_videobuf_code[dirac_videobuf_code_len++]=demux_getc(ds);
		memmove (dirac_videobuffer, dirac_videobuf_code, 5);
		dirac_videobuf_len = 5;
	}
	while (1)
	{
		int c;
		if(dirac_videobuf_code[0]==0x42 && dirac_videobuf_code[1]==0x42 && dirac_videobuf_code[2]==0x43 && dirac_videobuf_code[3]==0x44)
		{
			break;
		}
		++skipped;
		dirac_videobuffer[0] = dirac_videobuf_code[0]=dirac_videobuf_code[1];
		dirac_videobuffer[1] = dirac_videobuf_code[1]=dirac_videobuf_code[2];
		dirac_videobuffer[2] = dirac_videobuf_code[2]=dirac_videobuf_code[3];
		dirac_videobuffer[3] = dirac_videobuf_code[3]=dirac_videobuf_code[4];
		c = demux_getc(ds);
		if (c < 0 )
			return c;
		dirac_videobuffer[4] = dirac_videobuf_code[4]=c;
		dirac_videobuf_len = 5;
	}
	return dirac_videobuf_code[4];
}

/*
* Access unit header = 0x00
* End of sequence    = 0x10
* Auxiliary data     = 0x20
* Padding data       = 0x30
* Intra_Ref start    = 0x0C (using ac)
* Intra_NonRef start = 0x08 (using ac)
* Intra_Ref start    = 0x4C (not using ac)
* Intra_NonRef start = 0x48 (not using ac)
* Intra_Ref start    = 0xCC (low delay syntax)
* Intra_NonRef start = 0xC8 (low delay syntax)
* Inter_Ref_1Ref start = 0x0D
* Inter_Ref_2Ref start = 0x0E
* Inter_NonRef_1Ref start = 0x09
* Inter_NonRef_2Ref start = 0x0A
*/
#define FRAME_START(c) (((c) == 0x00) || ((c) == 0x10) || (((c)&0xF8) == 0x20) || ((c) == 0x30) || (((c)&0x08) == 0x08))

int dirac_read_video_frame (demux_stream_t *ds, unsigned char** start)
{
	int in_frame = 0;
	unsigned int shift = 0xffffffff;
	int msg_type = 0xFF;


	if (!dirac_videobuffer)
	{
		dirac_videobuffer = (unsigned char *)malloc(DIRAC_VIDEOBUFFER_SIZE);
		if(!dirac_videobuffer)
		{
			mp_msg(MSGT_DECVIDEO,MSGL_ERR,MSGTR_ShMemAllocFail);
			return 0;
		}
	}

	*start = dirac_videobuffer;

	/*while (msg_type != 0 && msg_type == 0xFF)*/
	msg_type = dirac_sync_video_packet(ds);

	if (msg_type == -1)
		return -1;


	/* find start of frame */
	while (!in_frame)
	{
		int byte;
		if (FRAME_START(msg_type))
		{
			in_frame = 1;
			break;
		}
	
		byte = demux_getc(ds);
		if (byte < 0)
		{
			dirac_videobuf_code_len = 0;
			if (dirac_videobuf_len)
				return dirac_videobuf_len;
			else
				return -1;
		}

		dirac_videobuffer[dirac_videobuf_len++] = byte;
		if (shift == 0x42424344)
		{
			if (FRAME_START(byte))
			{
				in_frame = 1;
				break;
			}
		}
		shift = (shift << 8 ) | byte;
	}

	/* find end of frame */
	shift = 0xffffffff;
	dirac_videobuf_code_len = 0;
	while (in_frame)
	{
		int byte;
		byte = demux_getc(ds);
		if (byte < 0)
			return dirac_videobuf_len;

		dirac_videobuffer[dirac_videobuf_len++] = byte;
		if (shift == 0x42424344)
		{
			if (byte != 0xFF)
			{
				in_frame = 0;
				dirac_videobuf_code_len = 5;
				dirac_videobuf_code[0] = 0x42;
				dirac_videobuf_code[1] = 0x42;
				dirac_videobuf_code[2] = 0x43;
				dirac_videobuf_code[3] = 0x44;
				dirac_videobuf_code[4] = byte;
				break;
			}
		}
		shift = (shift << 8 ) | byte;
	}
	//return dirac_videobuf_len - dirac_videobuf_code_len;
	return dirac_videobuf_len;
}

/* TODO: */
int dirac_skip_video_frame(demux_stream_t *ds)
{
	return 0;
}
